package dev.shadowsoffire.apotheosis.adventure.commands;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import com.mojang.brigadier.arguments.ArgumentType;
import com.mojang.brigadier.arguments.FloatArgumentType;
import com.mojang.brigadier.builder.ArgumentBuilder;
import com.mojang.brigadier.builder.LiteralArgumentBuilder;
import com.mojang.brigadier.builder.RequiredArgumentBuilder;
import com.mojang.brigadier.context.CommandContext;
import com.mojang.brigadier.exceptions.CommandSyntaxException;
import com.mojang.brigadier.suggestion.SuggestionProvider;
import com.mojang.brigadier.suggestion.SuggestionsBuilder;
import dev.shadowsoffire.apotheosis.adventure.affix.Affix;
import dev.shadowsoffire.apotheosis.adventure.affix.AffixHelper;
import dev.shadowsoffire.apotheosis.adventure.affix.AffixInstance;
import dev.shadowsoffire.apotheosis.adventure.affix.AffixRegistry;
import dev.shadowsoffire.apotheosis.adventure.loot.LootCategory;
import dev.shadowsoffire.apotheosis.adventure.loot.LootController;
import dev.shadowsoffire.apotheosis.adventure.loot.LootRarity;
import dev.shadowsoffire.attributeslib.api.AttributeHelper;
import dev.shadowsoffire.placebo.reload.DynamicHolder;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.CompletableFuture;
import java.util.stream.Stream;
import net.minecraft.ChatFormatting;
import net.minecraft.commands.CommandSourceStack;
import net.minecraft.commands.Commands;
import net.minecraft.commands.SharedSuggestionProvider;
import net.minecraft.commands.arguments.ResourceLocationArgument;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.HoverEvent;
import net.minecraft.network.chat.MutableComponent;
import net.minecraft.network.chat.Style;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.item.ItemStack;

public class AffixCommand {
  public static final SuggestionProvider<CommandSourceStack> SUGGEST_AFFIX;
  
  public static final SuggestionProvider<CommandSourceStack> SUGGEST_APPLICABLE_AFFIX;
  
  public static final SuggestionProvider<CommandSourceStack> SUGGEST_AFFIX_ON_ITEM;
  
  static {
    SUGGEST_AFFIX = ((ctx, builder) -> SharedSuggestionProvider.m_82981_(AffixRegistry.INSTANCE.getKeys().stream().map(ResourceLocation::toString), builder));
    SUGGEST_APPLICABLE_AFFIX = ((ctx, builder) -> {
        Entity entity = ((CommandSourceStack)ctx.getSource()).m_81373_();
        if (entity instanceof LivingEntity) {
          LivingEntity living = (LivingEntity)entity;
          ItemStack held = living.m_21205_();
          if (!held.m_41619_()) {
            LootCategory cat = LootCategory.forItem(held);
            DynamicHolder<LootRarity> rarity = AffixHelper.getRarity(held);
            if (!cat.isNone() && rarity.isBound()) {
              Objects.requireNonNull(AffixRegistry.INSTANCE);
              Stream<String> suggestions = AffixRegistry.INSTANCE.getValues().stream().filter(()).map(AffixRegistry.INSTANCE::getKey).map(ResourceLocation::toString);
              return SharedSuggestionProvider.m_82981_(suggestions, builder);
            } 
          } 
        } 
        return SharedSuggestionProvider.m_82970_(Collections.emptyList(), builder);
      });
    SUGGEST_AFFIX_ON_ITEM = ((ctx, builder) -> {
        Entity entity = ((CommandSourceStack)ctx.getSource()).m_81373_();
        if (entity instanceof LivingEntity) {
          LivingEntity living = (LivingEntity)entity;
          ItemStack held = living.m_21205_();
          if (!held.m_41619_()) {
            Map<DynamicHolder<? extends Affix>, AffixInstance> affixes = AffixHelper.getAffixes(held);
            return SharedSuggestionProvider.m_82981_(affixes.keySet().stream().map(DynamicHolder::getId).map(ResourceLocation::toString), builder);
          } 
        } 
        return SharedSuggestionProvider.m_82970_(Collections.emptyList(), builder);
      });
  }
  
  public static void register(LiteralArgumentBuilder<CommandSourceStack> root) {
    LiteralArgumentBuilder<CommandSourceStack> builder = (LiteralArgumentBuilder<CommandSourceStack>)Commands.m_82127_("affix").requires(c -> c.m_6761_(2));
    builder.then(
        Commands.m_82127_("apply")
        .then(((RequiredArgumentBuilder)Commands.m_82129_("affix", (ArgumentType)ResourceLocationArgument.m_106984_()).suggests(SUGGEST_APPLICABLE_AFFIX)
          .then(Commands.m_82129_("level", (ArgumentType)FloatArgumentType.floatArg(0.0F, 1.0F))
            .executes(c -> applyAffix(c, ResourceLocationArgument.m_107011_(c, "affix"), FloatArgumentType.getFloat(c, "level")))))
          .executes(c -> applyAffix(c, ResourceLocationArgument.m_107011_(c, "affix"), (((CommandSourceStack)c.getSource()).m_81372_()).f_46441_.m_188501_()))));
    builder.then(
        Commands.m_82127_("list")
        .executes(dev.shadowsoffire.apotheosis.adventure.commands.AffixCommand::listAffixes));
    builder.then(
        Commands.m_82127_("list_alternatives")
        .then(Commands.m_82129_("affix", (ArgumentType)ResourceLocationArgument.m_106984_()).suggests(SUGGEST_AFFIX_ON_ITEM)
          .executes(c -> listAlternatives(c, ResourceLocationArgument.m_107011_(c, "affix")))));
    root.then((ArgumentBuilder)builder);
  }
  
  public static int applyAffix(CommandContext<CommandSourceStack> c, ResourceLocation affixId, float level) {
    DynamicHolder<Affix> afx = AffixRegistry.INSTANCE.holder(affixId);
    if (!afx.isBound())
      return fail(c, "Unknown affix: " + affixId, -1); 
    Entity entity = ((CommandSourceStack)c.getSource()).m_81373_();
    if (entity instanceof LivingEntity) {
      LivingEntity living = (LivingEntity)entity;
      ItemStack held = living.m_21205_();
      if (held.m_41619_())
        return fail(c, "The target entity must have an item in their main hand.", -2); 
      DynamicHolder<LootRarity> rarity = AffixHelper.getRarity(held);
      if (!rarity.isBound())
        return fail(c, "The target item must have a set rarity.", -3); 
      LootCategory cat = LootCategory.forItem(held);
      if (cat.isNone())
        return fail(c, "The target item must have a valid loot category", -4); 
      if (!((Affix)afx.get()).canApplyTo(held, cat, (LootRarity)rarity.get()))
        return fail(c, "The selected affix cannot be applied to the target item.", -5); 
      AffixHelper.applyAffix(held, new AffixInstance(afx, held, rarity, level));
      ((CommandSourceStack)c.getSource()).m_288197_(() -> Component.m_237110_("Successfully applied affix %s with level %s to %s", new Object[] { affixId, Float.valueOf(level), held.m_41611_() }), true);
      return 0;
    } 
    return fail(c, "/apoth affix must be executed by a living entity.", -10);
  }
  
  public static int listAffixes(CommandContext<CommandSourceStack> c) {
    Entity entity = ((CommandSourceStack)c.getSource()).m_81373_();
    if (entity instanceof LivingEntity) {
      LivingEntity living = (LivingEntity)entity;
      ItemStack held = living.m_21205_();
      if (held.m_41619_())
        return fail(c, "The target entity must have an item in their main hand.", -2); 
      DynamicHolder<LootRarity> rarity = AffixHelper.getRarity(held);
      if (!rarity.isBound())
        return fail(c, "The target item must have a set rarity.", -3); 
      Map<DynamicHolder<? extends Affix>, AffixInstance> affixes = AffixHelper.getAffixes(held);
      ((CommandSourceStack)c.getSource()).m_243053_((Component)Component.m_237110_("Affixes present on %s:", new Object[] { held.m_41611_() }));
      affixes.forEach((afx, inst) -> {
            MutableComponent name = Component.m_237110_("[%s]", new Object[] { inst.getName(true) });
            name.m_6270_(Style.f_131099_.m_131140_(ChatFormatting.YELLOW).m_131144_(new HoverEvent(HoverEvent.Action.f_130831_, inst.getAugmentingText())));
            ((CommandSourceStack)c.getSource()).m_243053_((Component)Component.m_237110_("%s - %s%%", new Object[] { name, Affix.fmt(100.0F * inst.level()) }));
          });
      return 0;
    } 
    return fail(c, "/apoth affix must be executed by a living entity.", -10);
  }
  
  public static int listAlternatives(CommandContext<CommandSourceStack> c, ResourceLocation affixId) {
    DynamicHolder<Affix> afx = AffixRegistry.INSTANCE.holder(affixId);
    if (!afx.isBound())
      return fail(c, "Unknown affix: " + affixId, -1); 
    Entity entity = ((CommandSourceStack)c.getSource()).m_81373_();
    if (entity instanceof LivingEntity) {
      LivingEntity living = (LivingEntity)entity;
      ItemStack held = living.m_21205_();
      if (held.m_41619_())
        return fail(c, "The target entity must have an item in their main hand.", -2); 
      DynamicHolder<LootRarity> rarity = AffixHelper.getRarity(held);
      if (!rarity.isBound())
        return fail(c, "The target item must have a set rarity.", -3); 
      Map<DynamicHolder<? extends Affix>, AffixInstance> affixes = AffixHelper.getAffixes(held);
      if (!affixes.containsKey(afx))
        return fail(c, "The target item does not contain the selected affix.", -4); 
      List<DynamicHolder<? extends Affix>> alternatives = LootController.getAvailableAffixes(held, (LootRarity)rarity.get(), affixes.keySet(), ((Affix)afx.get()).getType());
      ((CommandSourceStack)c.getSource()).m_243053_((Component)Component.m_237110_("Possible alternatives to %s:", new Object[] { ((Affix)afx.get()).getName(true) }));
      alternatives.forEach(a -> {
            MutableComponent name = Component.m_237110_("[%s]", new Object[] { ((Affix)a.get()).getName(true) });
            Component augTxt = ((Affix)a.get()).getAugmentingText(held, (LootRarity)rarity.get(), 0.5F);
            name.m_6270_(Style.f_131099_.m_131140_(ChatFormatting.YELLOW).m_131144_(new HoverEvent(HoverEvent.Action.f_130831_, augTxt)));
            ((CommandSourceStack)c.getSource()).m_243053_((Component)AttributeHelper.list().m_7220_((Component)name));
          });
      return 0;
    } 
    return fail(c, "/apoth affix must be executed by a living entity.", -10);
  }
  
  public static int fail(CommandContext<CommandSourceStack> c, String msg, int code) {
    ((CommandSourceStack)c.getSource()).m_81352_((Component)Component.m_237115_(msg));
    return code;
  }
}
